<?php
/**
 * @file
 * Defines the administration forms for managing inline documentation.
 */

/**
 * List all inline documentation form.
 *
 * @return string
 *   HTML table with inline documentation entry.
 */
function inline_documentation_overview() {
  // Configure the table header columns.
  $header = array(
    array(
      'data' => t('Title'),
      'field' => 'title',
    ),
    array(
      'data' => t('Author'),
    ),
    array(
      'data' => t('Status'),
      'field' => 'status',
    ),
    array(
      'data' => t('Created'),
      'field' => 'created',
    ),
    array(
      'data' => t('Updated'),
      'field' => 'changed',
      'sort' => 'desc',
    ),
    array(
      'data' => t('Operations'),
    ),
  );


  $select = db_select('inline_documentation', 'i')
    ->extend('PagerDefault')
    ->extend('TableSort');

  $select->fields('i', array('id', 'title', 'uid', 'created', 'changed', 'status'))
    ->limit(10)
    ->orderByHeader($header);

  $results = $select->execute();

  $rows = array();
  foreach ($results as $row) {
    $author = user_load($row->uid);

    $operations = array(
      l(t('Edit'), "admin/structure/inline_documentation/{$row->id}/edit"),
      l(t('Delete'), "admin/structure/inline_documentation/{$row->id}/delete"),
    );
    $rows[] = array(
      l($row->title, "admin/structure/inline_documentation/{$row->id}"),
      $author->name,
      !empty($row->status) ? 'published' : 'unpublished',
      format_date($row->created, 'short'),
      format_date($row->changed, 'short'),
      theme('item_list', array(
        'items' => $operations,
        'attributes' => array(
          'class' => array('links', 'inline'),
        ),
      )),
    );
  }

  $output = theme('table', array(
    'header' => $header,
    'rows' => $rows,
  ));

  $output .= theme('pager');

  return $output;
}

/**
 * Create or edit an inline documentation form.
 *
 * @param array $form
 *   The form being used to edit the node.
 * @param array $form_state
 *   The form state array.
 * @param array $edit
 *   The object to update.
 *
 * @return mixed
 *   Renderable array containing a form.
 */
function inline_documentation_edit_form($form, &$form_state, $edit = array()) {
  // During initial form build, add the entity to the form state for use
  // during form building and processing. During a rebuild, use what is in
  // the form state.
  if (!isset($form_state['inline_documentation'])) {
    // Convert the edit variable to an object.
    $inline_documentation = is_object($edit) ? $edit : (object) $edit;

    // Create array with default values.
    $default = array(
      'title' => '',
      'pages' => '',
      'element' => '',
      'status' => 1,
      'weight' => 0,
    );

    // Add the current page to the defaults if the current page isn't an admin
    // page.
    if (!path_is_admin(current_path())) {
      $default['pages'] = current_path();
    }

    // Iterate through default values.
    foreach ($default as $key => $value) {
      // If the property is missing.
      if (!isset($inline_documentation->{$key})) {
        // Then add the default value.
        $inline_documentation->{$key} = $value;
      }
    }
    // Assign inline_documentation to the form state.
    $form_state['inline_documentation'] = $inline_documentation;
  }
  else {
    // Retrieve the inline_documentation from the form state.
    $inline_documentation = $form_state['inline_documentation'];
  }
  // Add inline_documentation to the form.
  $form['#inline_documentation'] = $form_state['inline_documentation'];

  // Load the extra fields information so set the correct order.
  $extrafields = field_info_extra_fields('inline_documentation', 'inline_documentation', 'form');

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#description' => t('The title of the inline documentation.'),
    '#default_value' => $inline_documentation->title,
    '#maxlength' => 255,
    '#required' => TRUE,
    '#weight' => isset($extrafields['title']['weight']) ? $extrafields['title']['weight'] : 0,
  );
  $form['pages'] = array(
    '#type' => 'textarea',
    '#title' => t('Pages'),
    '#description' => t("Specify pages by using their paths. Enter one path per line. The '*' character is a wildcard."),
    '#default_value' => $inline_documentation->pages,
    '#required' => TRUE,
    '#weight' => isset($extrafields['pages']['weight']) ? $extrafields['pages']['weight'] : 2,
  );
  $form['element_wrapper'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'class' => array('element-wrapper'),
    ),
    '#weight' => isset($extrafields['element']['weight']) ? $extrafields['element']['weight'] : 3,
  );
  $form['element_wrapper']['element'] = array(
    '#type' => 'textfield',
    '#title' => t('Element'),
    '#description' => t('The element to attach the inline documentation to.'),
    '#default_value' => $inline_documentation->element,
    '#maxlength' => 255,
  );

  // Add the current page to the defaults if the current page isn't an admin
  // page.
  if (!path_is_admin(current_path())) {
    $form['element_wrapper']['selector'] = array(
      '#markup' => l('<span>' . t('Select element') . '</span>', current_path(), array(
        'html' => TRUE,
        'attributes' => array(
          'class' => array('inline-documentation-select-element'),
        ),
        'fragment' => 'inline-documentation',
      )),
    );
  }

  $form['status'] = array(
    '#type' => 'checkbox',
    '#title' => t('Published'),
    '#description' => t('Unpublished inline documentation will not be displayed on the website.<br />Note that users with the permission to edit this inline_documentation will still be able to see this inline_documentation.'),
    '#default_value' => $inline_documentation->status,
    '#weight' => isset($extrafields['status']['weight']) ? $extrafields['status']['weight'] : 4,
  );

  // Add supported actions for this form.
  $form['actions'] = array(
    '#type' => 'actions',
  );
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  // Add fields to the form.
  field_attach_form('inline_documentation', $inline_documentation, $form, $form_state);

  // Return the generated form.
  return $form;
}

/**
 * Confirm inline documentation deletion form.
 *
 * @param array $form
 *   The form being used to edit the node.
 * @param array $form_state
 *   The form state array.
 * @param object $inline_documentation
 *   The object to delete.
 *
 * @return mixed
 *   Renderable array containing a form.
 */
function inline_documentation_form_confirm_delete($form, &$form_state, $inline_documentation) {
  // Always provide entity id in the same form key as in the entity edit form.
  $form['id'] = array('#type' => 'value', '#value' => $inline_documentation->id);
  // Add inline_documentation to the form.
  $form['#inline_documentation'] = $inline_documentation;
  $form['type'] = array('#type' => 'value', '#value' => 'inline_documentation');
  $form['title'] = array('#type' => 'value', '#value' => $inline_documentation->title);
  return confirm_form(
      $form,
      t('Are you sure you want to delete the inline documentation %title?', array('%title' => $inline_documentation->title)),
      'admin/structure/inline_documentation',
      t('This action cannot be undone.'),
      t('Delete'),
      t('Cancel')
  );
}

/**
 * Form validation handler for 'inline_documentation_form'.
 */
function inline_documentation_edit_form_validate($form, &$form_state) {
  // Allow the entity fields to be validated.
  entity_form_field_validate('inline_documentation', $form, $form_state);
}

/**
 * Form submit handler for 'inline_documentation_edit_form'.
 */
function inline_documentation_edit_form_submit($form, &$form_state) {
  // Retrieve the inline_documentation.
  $inline_documentation = $form_state['inline_documentation'];

  // Submit the inline_documentation entity.
  entity_form_submit_build_entity('inline_documentation', $inline_documentation, $form, $form_state);
  // Evaluate the save status.
  switch (inline_documentation_save($inline_documentation)) {
    case SAVED_NEW:
      drupal_set_message(t('Created new inline documentation %title.', array('%title' => $inline_documentation->title)));
      watchdog('inline_documentation', 'Created new inline documentation %title.', array('%title' => $inline_documentation->title), WATCHDOG_NOTICE, l(t('edit'), "admin/structure/inline_documentation/{$inline_documentation->id}/edit"));
      break;

    case SAVED_UPDATED:
      drupal_set_message(t('Updated inline documentation %title.', array('%title' => $inline_documentation->title)));
      watchdog('inline_documentation', 'Updated inline documentation %title.', array('%title' => $inline_documentation->title), WATCHDOG_NOTICE, l(t('edit'), "admin/structure/inline_documentation/{$inline_documentation->id}/edit"));
      break;
  }
  // Add inline_documentation information to the form state.
  $form_state['values']['id'] = $inline_documentation->id;
  $form_state['id'] = $inline_documentation->id;
  // Redirect to the 'admin/structure/inline_documentation' page.
  $form_state['redirect'] = 'admin/structure/inline_documentation';

  // Redirect back to the page where this form is called on when the current
  // page isn't an admin page.
  if (!path_is_admin(current_path())) {
    $form_state['redirect'] = current_path();
  }
}

/**
 * Form submit handler for 'inline_documentation_form_confirm_delete'.
 */
function inline_documentation_form_confirm_delete_submit($form, &$form_state) {
  // Delete the inline_documentation.
  inline_documentation_delete($form_state['values']['id']);
  drupal_set_message(t('Deleted inline documentation %title', array('%title' => $form_state['values']['title'])));
  watchdog('inline_documentation', 'Deleted inline documentation %title', array('%title' => $form_state['values']['title']), WATCHDOG_NOTICE);
  $form_state['redirect'] = 'admin/structure/inline_documentation';
  // Clear the page and block caches to avid stale data.
  cache_clear_all();
}
